<script type="text/javascript">//<![CDATA[
XHR.poll(3, '<%=url([[admin]], [[services]], [[aliyundrive-webdav]], [[status]])%>', null,
	function(x, data) {
		var tb = document.getElementById('aliyundrive-webdav_status');
		if (data && tb) {
			if (data.running) {
				tb.innerHTML = '<em><b><font color=green>' + data.application + '<%:RUNNING%></font></b></em>';
			} else {
				tb.innerHTML = '<em><b><font color=red>' + data.application + '<%:NOT RUNNING%></font></b></em>';
			}
		}
	}
);
//]]>
</script>
<style>.mar-10 {margin-left: 50px; margin-right: 10px;}</style>
<fieldset class="cbi-section">
	<p id="aliyundrive-webdav_status">
		<em><%:Collecting data...%></em>
	</p>
</fieldset>
